@@ -12,7 +12,7 @@ pub trait Matmul<T> {
     /// └───────┘        │ 1 2 │  └─────┘
     ///                  └─────┘
     /// ```
-    fn matmul(self, other: T) -> Self::Output;
+    fn matmul(&self, other: &T) -> Self::Output;
 }
 
 // - L: Rows of `self` and rows of `Self::Output`
@@ -31,8 +31,8 @@ fn matmul<T: Debug + Mul<Output = T> + AddAssign + Copy + Debug>(
     debug_assert_eq!(c.len(), l_len * n_len);
 
     for l in 0..l_len {
-        for n in 0..n_len {
-            for m in 0..m_len {
+        for m in 0..m_len {
+            for n in 0..n_len {
                 let (i, j, k) = (l * n_len + n, l * m_len + m, m * n_len + n);
                 c[i] += a[j] * b[k];
                 // c[l][n] += a[l][m] * b[m][n];
@@ -49,7 +49,7 @@ where
     [(); M * N]:,
 {
     type Output = MatrixDxS<T, N>;
-    fn matmul(self, other: MatrixSxS<T, M, N>) -> Self::Output {
+    fn matmul(&self, other: &MatrixSxS<T, M, N>) -> Self::Output {
         assert_eq!(self.columns, M, "Non-matching columns to rows");
 
         let l_len = self.rows;
@@ -62,7 +62,7 @@ impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T>, const N: usize>
     Matmul<MatrixDxS<T, N>> for MatrixDxD<T>
 {
     type Output = MatrixDxS<T, N>;
-    fn matmul(self, other: MatrixDxS<T, N>) -> Self::Output {
+    fn matmul(&self, other: &MatrixDxS<T, N>) -> Self::Output {
         assert_eq!(self.columns, other.rows, "Non-matching columns to rows");
 
         let (l, m) = (self.rows, self.columns);
@@ -75,7 +75,7 @@ impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
     Matmul<MatrixSxD<T, M>> for MatrixDxD<T>
 {
     type Output = MatrixDxD<T>;
-    fn matmul(self, other: MatrixSxD<T, M>) -> Self::Output {
+    fn matmul(&self, other: &MatrixSxD<T, M>) -> Self::Output {
         let (l, n) = (self.rows, other.columns);
         let mut data = vec![Default::default(); l * n];
         matmul((l, n, M), (&self.data, &other.data, &mut data));
@@ -90,7 +90,7 @@ impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug>
     for MatrixDxD<T>
 {
     type Output = MatrixDxD<T>;
-    fn matmul(self, other: MatrixDxD<T>) -> Self::Output {
+    fn matmul(&self, other: &MatrixDxD<T>) -> Self::Output {
         assert_eq!(self.columns, other.rows, "Non-matching columns to rows");
 
         let (l, m, n) = (self.rows, self.columns, other.columns);
@@ -111,7 +111,7 @@ where
     [(); M * N]:,
 {
     type Output = MatrixDxS<T, N>;
-    fn matmul(self, other: MatrixSxS<T, M, N>) -> Self::Output {
+    fn matmul(&self, other: &MatrixSxS<T, M, N>) -> Self::Output {
         let l = self.rows;
         let mut data = vec![Default::default(); l * N];
         matmul((l, N, M), (&self.data, &other.data, &mut data));
@@ -122,7 +122,7 @@ impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T>, const M: usize, co
     Matmul<MatrixDxS<T, N>> for MatrixDxS<T, M>
 {
     type Output = MatrixDxS<T, N>;
-    fn matmul(self, other: MatrixDxS<T, N>) -> Self::Output {
+    fn matmul(&self, other: &MatrixDxS<T, N>) -> Self::Output {
         let l = self.rows;
         let mut data = vec![Default::default(); l * N];
         matmul((l, N, M), (&self.data, &other.data, &mut data));
@@ -133,7 +133,7 @@ impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
     Matmul<MatrixSxD<T, M>> for MatrixDxS<T, M>
 {
     type Output = MatrixDxD<T>;
-    fn matmul(self, other: MatrixSxD<T, M>) -> Self::Output {
+    fn matmul(&self, other: &MatrixSxD<T, M>) -> Self::Output {
         let (l, n) = (self.rows, other.columns);
         let mut data = vec![Default::default(); l * n];
         matmul((l, n, M), (&self.data, &other.data, &mut data));
@@ -148,7 +148,7 @@ impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
     Matmul<MatrixDxD<T>> for MatrixDxS<T, M>
 {
     type Output = MatrixDxD<T>;
-    fn matmul(self, other: MatrixDxD<T>) -> Self::Output {
+    fn matmul(&self, other: &MatrixDxD<T>) -> Self::Output {
         assert_eq!(M, other.rows, "Non-matching columns to rows");
 
         let (l, n) = (self.rows, other.columns);
@@ -174,7 +174,7 @@ where
     [(); M * N]:,
 {
     type Output = MatrixSxS<T, L, N>;
-    fn matmul(self, other: MatrixSxS<T, M, N>) -> Self::Output {
+    fn matmul(&self, other: &MatrixSxS<T, M, N>) -> Self::Output {
         let mut data = [Default::default(); L * N];
         matmul((L, N, M), (&self.data, &other.data, &mut data));
         Self::Output { data }
@@ -186,7 +186,7 @@ where
     [(); L * N]:,
 {
     type Output = MatrixSxS<T, L, N>;
-    fn matmul(self, other: MatrixDxS<T, N>) -> Self::Output {
+    fn matmul(&self, other: &MatrixDxS<T, N>) -> Self::Output {
         assert_eq!(self.columns, other.rows, "Non-matching columns to rows");
 
         let m = self.columns;
@@ -202,7 +202,7 @@ impl<
     > Matmul<MatrixSxD<T, M>> for MatrixSxD<T, L>
 {
     type Output = MatrixSxD<T, L>;
-    fn matmul(self, other: MatrixSxD<T, M>) -> Self::Output {
+    fn matmul(&self, other: &MatrixSxD<T, M>) -> Self::Output {
         let n = other.columns;
         let mut data = vec![Default::default(); L * n];
         matmul((L, n, M), (&self.data, &other.data, &mut data));
@@ -213,7 +213,7 @@ impl<T: Debug + Default + Copy + AddAssign + Mul<Output = T> + std::fmt::Debug,
     Matmul<MatrixDxD<T>> for MatrixSxD<T, L>
 {
     type Output = MatrixSxD<T, L>;
-    fn matmul(self, other: MatrixDxD<T>) -> Self::Output {
+    fn matmul(&self, other: &MatrixDxD<T>) -> Self::Output {
         assert_eq!(self.columns, other.rows, "Non-matching columns to rows");
 
         let (n, m) = (other.columns, self.columns);
@@ -236,7 +236,7 @@ where
     [(); L * N]:,
 {
     type Output = MatrixSxS<T, L, N>;
-    fn matmul(self, other: MatrixSxS<T, M, N>) -> Self::Output {
+    fn matmul(&self, other: &MatrixSxS<T, M, N>) -> Self::Output {
         let mut data = [Default::default(); L * N];
         matmul((L, N, M), (&self.data, &other.data, &mut data));
         Self::Output { data }
@@ -253,7 +253,7 @@ where
     [(); L * N]:,
 {
     type Output = MatrixSxS<T, L, N>;
-    fn matmul(self, other: MatrixDxS<T, N>) -> Self::Output {
+    fn matmul(&self, other: &MatrixDxS<T, N>) -> Self::Output {
         let mut data = [Default::default(); L * N];
         matmul((L, N, M), (&self.data, &other.data, &mut data));
         Self::Output { data }
@@ -268,7 +268,7 @@ where
     [(); L * M]:,
 {
     type Output = MatrixSxD<T, L>;
-    fn matmul(self, other: MatrixSxD<T, M>) -> Self::Output {
+    fn matmul(&self, other: &MatrixSxD<T, M>) -> Self::Output {
         let n = other.columns;
         let mut data = vec![Default::default(); L * n];
         matmul((L, n, M), (&self.data, &other.data, &mut data));
@@ -284,7 +284,7 @@ where
     [(); L * M]:,
 {
     type Output = MatrixSxD<T, L>;
-    fn matmul(self, other: MatrixDxD<T>) -> Self::Output {
+    fn matmul(&self, other: &MatrixDxD<T>) -> Self::Output {
         assert_eq!(M, other.rows, "Non-matching columns to rows");
 
         let n = other.columns;
@@ -305,7 +305,7 @@ mod tests {
     fn dxd_dxd() {
         let a = MatrixDxD::try_from(vec![vec![1, 3, 5], vec![2, 4, 6]]).unwrap();
         let b = MatrixDxD::try_from(vec![vec![7, 10], vec![8, 11], vec![9, 12]]).unwrap();
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixDxD::try_from(vec![vec![76, 103], vec![100, 136]]).unwrap();
         assert_eq!(c, d);
     }
@@ -313,7 +313,7 @@ mod tests {
     fn dxd_dxs() {
         let a = MatrixDxD::try_from(vec![vec![1, 3, 5], vec![2, 4, 6]]).unwrap();
         let b = MatrixDxS::from(vec![[7, 10], [8, 11], [9, 12]]);
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixDxS::from(vec![[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -321,7 +321,7 @@ mod tests {
     fn dxd_sxd() {
         let a = MatrixDxD::try_from(vec![vec![1, 3, 5], vec![2, 4, 6]]).unwrap();
         let b = MatrixSxD::try_from([vec![7, 10], vec![8, 11], vec![9, 12]]).unwrap();
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxD::try_from([vec![76, 103], vec![100, 136]]).unwrap();
         assert_eq!(c, d);
     }
@@ -329,7 +329,7 @@ mod tests {
     fn dxd_sxs() {
         let a = MatrixDxD::try_from(vec![vec![1, 3, 5], vec![2, 4, 6]]).unwrap();
         let b = MatrixSxS::from([[7, 10], [8, 11], [9, 12]]);
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxS::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -339,7 +339,7 @@ mod tests {
     fn dxs_dxs() {
         let a = MatrixDxS::from(vec![[1, 3, 5], [2, 4, 6]]);
         let b = MatrixDxS::from(vec![[7, 10], [8, 11], [9, 12]]);
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixDxS::from(vec![[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -347,7 +347,7 @@ mod tests {
     fn dxs_sxd() {
         let a = MatrixDxS::from(vec![[1, 3, 5], [2, 4, 6]]);
         let b = MatrixSxD::try_from([vec![7, 10], vec![8, 11], vec![9, 12]]).unwrap();
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxS::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -355,7 +355,7 @@ mod tests {
     fn dxs_dxd() {
         let a = MatrixDxS::from(vec![[1, 3, 5], [2, 4, 6]]);
         let b = MatrixDxD::try_from(vec![vec![7, 10], vec![8, 11], vec![9, 12]]).unwrap();
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixDxS::from(vec![[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -363,7 +363,7 @@ mod tests {
     fn dxs_sxs() {
         let a = MatrixDxS::from(vec![[1, 3, 5], [2, 4, 6]]);
         let b = MatrixSxS::from([[7, 10], [8, 11], [9, 12]]);
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxS::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -373,7 +373,7 @@ mod tests {
     fn sxd_sxd() {
         let a = MatrixSxD::try_from([vec![1, 3, 5], vec![2, 4, 6]]).unwrap();
         let b = MatrixSxD::try_from([vec![7, 10], vec![8, 11], vec![9, 12]]).unwrap();
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxD::try_from([vec![76, 103], vec![100, 136]]).unwrap();
         assert_eq!(c, d);
     }
@@ -381,7 +381,7 @@ mod tests {
     fn sxd_dxs() {
         let a = MatrixSxD::try_from([vec![1, 3, 5], vec![2, 4, 6]]).unwrap();
         let b = MatrixDxS::from(vec![[7, 10], [8, 11], [9, 12]]);
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxS::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -389,7 +389,7 @@ mod tests {
     fn sxd_sxs() {
         let a = MatrixSxD::try_from([vec![1, 3, 5], vec![2, 4, 6]]).unwrap();
         let b = MatrixSxS::from([[7, 10], [8, 11], [9, 12]]);
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxS::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -397,7 +397,7 @@ mod tests {
     fn sxd_dxd() {
         let a = MatrixSxD::try_from([vec![1, 3, 5], vec![2, 4, 6]]).unwrap();
         let b = MatrixDxD::try_from(vec![vec![7, 10], vec![8, 11], vec![9, 12]]).unwrap();
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxD::try_from([vec![76, 103], vec![100, 136]]).unwrap();
         assert_eq!(c, d);
     }
@@ -405,17 +405,17 @@ mod tests {
     // --------------------------------------------------
     #[test]
     fn sxs_sxs() {
-        let a = MatrixSxS::from([[1, 3, 5], [2, 4, 6]]);
-        let b = MatrixSxS::from([[7, 10], [8, 11], [9, 12]]);
-        let c = a.matmul(b);
-        let d = MatrixSxS::from([[76, 103], [100, 136]]);
+        let a = MatrixSxS::<i32, 2, 3>::from([[1, 3, 5], [2, 4, 6]]);
+        let b = MatrixSxS::<i32, 3, 2>::from([[7, 10], [8, 11], [9, 12]]);
+        let c = a.matmul(&b);
+        let d = MatrixSxS::<i32, 2, 2>::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
     #[test]
     fn sxs_dxs() {
         let a = MatrixSxS::from([[1, 3, 5], [2, 4, 6]]);
         let b = MatrixDxS::from(vec![[7, 10], [8, 11], [9, 12]]);
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxS::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -423,7 +423,7 @@ mod tests {
     fn sxs_sxd() {
         let a = MatrixSxS::from([[1, 3, 5], [2, 4, 6]]);
         let b = MatrixSxD::try_from([vec![7, 10], vec![8, 11], vec![9, 12]]).unwrap();
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxS::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
@@ -431,7 +431,7 @@ mod tests {
     fn sxs_dxd() {
         let a = MatrixSxS::from([[1, 3, 5], [2, 4, 6]]);
         let b = MatrixDxD::try_from(vec![vec![7, 10], vec![8, 11], vec![9, 12]]).unwrap();
-        let c = a.matmul(b);
+        let c = a.matmul(&b);
         let d = MatrixSxS::from([[76, 103], [100, 136]]);
         assert_eq!(c, d);
     }
