@@ -344,10 +344,10 @@ mod tests {
     // --------------------------------------------------
     #[test]
     fn sxs_sxs() {
-        let a = MatrixSxS::from([[1., 2., 3.], [4., 5., 6.]]);
-        let b = MatrixSxS::from([[7., 8., 9.], [10., 11., 12.]]);
+        let a = MatrixSxS::<f32, 2, 3>::from([[1., 2., 3.], [4., 5., 6.]]);
+        let b = MatrixSxS::<f32, 2, 3>::from([[7., 8., 9.], [10., 11., 12.]]);
         let c = a * b;
-        let d = MatrixSxS::from([[7., 16., 27.], [40., 55., 72.]]);
+        let d = MatrixSxS::<f32, 2, 3>::from([[7., 16., 27.], [40., 55., 72.]]);
         assert_eq!(c, d);
     }
     #[test]
